//
// image_component.h
//
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef NTK_GUI_image_component_H
# define NTK_GUI_image_component_H

# include <ntk/ntk.h>

class ImageComponent : public juce::Component
{
  public:
    ImageComponent()
      : m_image(0)
    {}

    juce_UseDebuggingNewOperator

    virtual ~ImageComponent()
    {
      delete m_image;
    }

  public:
    void setImage(Image* im)
    {
      if (m_image) delete m_image;
      m_image = im;
      repaint();
    }

  public:
    virtual void mouseDown(const MouseEvent& e)
    {
      if (!e.mods.isRightButtonDown() || !m_image)
        return;

      PopupMenu m;
      m.addItem(1, "Save Image As ...");
      const int result = m.show();

      if (result == 0)
      {
        // user dismissed the menu without picking anything
      }
      else if (result == 1)
      {
        FileChooser chooser("Please select a pgm file to save into...",
                            File::getCurrentWorkingDirectory().getChildFile("image.pgm"),
                            "*.pgm");

        if (chooser.browseForFileToSave(true))
        {
          String filename = chooser.getResult().getFullPathName();
          ntk::vil_byte_image vil_im;
          ntk::juce_to_vil(*m_image, vil_im);
          if (!vil_save(vil_im, filename.toUTF8()))
          {
            AlertWindow::showMessageBox(AlertWindow::WarningIcon,
                                        "Error !",
                                        "Could not save image.",
                                        "Ok");
          }
        }
      }
    }

    virtual void paint(Graphics& g)
    {
      if (m_image)
      {
        // g.setImageResamplingQuality(Graphics::lowResamplingQuality);
        g.drawImageWithin(m_image,
                          0,
                          0,
                          getWidth(),
                          getHeight(),
                          RectanglePlacement::stretchToFit,
                          false);
      }
      else
      {
        g.fillAll(Colours::lightgrey);
      }
    }

  private:
    Image* m_image;
};

#endif // ndef NTK_GUI_image_component_H
