//
// algorithm.h
// 
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef NTK_utils_algorithm_H
# define NTK_utils_algorithm_H

# include <ntk/core.h>

namespace ntk
{

  /**
   *  Filter a set of SORTED objects. The match with the lowest score
   *  must be the first one. Then for each object, if there is another
   *  object with a higher score for which predicate is true, 
   *  the first object is removed.
   */
  template <typename outputT, typename inputT, typename U>
  void keep_maximal_among_sorted_objects(outputT output, 
                                         inputT begin, inputT end, 
                                         U predicate)
  {
    while (begin != end)
    {
      inputT tmp = begin; ++tmp;
      bool is_locally_maximal = true;
      while (tmp != end && is_locally_maximal)
      {
        if (predicate(*begin, *tmp))
          is_locally_maximal = false;
        ++tmp;
      }
      if (is_locally_maximal)
      {
        *output = *begin;
        ++output;
      }
      ++begin;
    }
  }

} // end of ntk

#endif // ndef NTK_algorithm_H
