//
// test-threadqueue.cc
// 
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#include <ntk/gui/imageviewer.h>
#include <ntk/utils/taskqueue.h>
#include <ntk/utils/runnable.h>
#include <ntk/utils/debug.h>
#include <ntk/utils/pointer.h>
#include <ntk/vxl/all.h>

#include <vnl/vnl_random.h>

#include <QApplication>
#include <QThread>
#include <QMetaType>

#include <unistd.h>

using namespace ntk;

static QMutex mutex;
static QMutex mutex2;

class TestTask : public Runnable
{
  public:
    static int id;
    int m_id;
  
  public:
    TestTask(TaskQueue& q)
      : m_queue(q)
    {
      mutex.lock();
      m_id = id++;
      mutex.unlock();
    }
  
  public:
    virtual void run()
    {
      mutex2.lock();
      std::cerr << "Starting task " << m_id << std::endl;
      mutex2.unlock();

      usleep((unsigned)((rand()*999) / RAND_MAX));
     
      mutex2.lock();
      if (id < 10000)
      {
        m_queue.enqueueTask(boost::shared_ptr<TestTask>(new TestTask(m_queue)));
        m_queue.enqueueTask(boost::shared_ptr<TestTask>(new TestTask(m_queue)));
      }
      mutex2.unlock();
      
      mutex2.lock();
      std::cerr << "Ending task " << m_id << std::endl;
      mutex2.unlock();
    }
    
  private:
    TaskQueue& m_queue;
};
int TestTask::id = 0;

int main(int argc, char** argv)
{
  QCoreApplication app(argc, argv);
  
  ntk::ntk_debug_level = 1;  
  
  if (1)
  {
    TaskQueue p(20);
    p.enqueueTask(boost::shared_ptr<TestTask>(new TestTask(p)));
    p.waitUntilWorkFinished();
  }
  
  if (0)
  {
    TaskQueue p(32);
    boost::shared_ptr<TestTask> t1 (new TestTask(p));
    p.enqueueTask(t1); p.enqueueTask(t1); 
    for (int i = 0; i < 100; ++i)
    {
      if ((i % 200) == 0)
        p.enqueueTask(t1);
      else
        p.enqueueTask(boost::shared_ptr<TestTask>(new TestTask(p)));    
    }
  }
}
