//
// test-xml.cc
//
// Author:  Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#include <ntk/utils/xml_serializable.h>
#include <ntk/utils/iostream.h>

using namespace ntk;

class TestXml : public ntk::XmlSerializable
{
  public:
    virtual ~TestXml() {}

    virtual QDomElement toXml(QDomElement element) const
    {
      addXmlAttribute(element, "value", value);
      return element;
    }

    virtual bool fromXml(const QDomElement& element, const QVariant& data = QVariant())
    {
      setFromXmlAttribute(element, "value", value);
      return true;
    }

    double value;
};

QString test_write()
{
  TestXml xml;
  xml.value = 42;
  QDomDocument doc ("TestXml");
  doc.appendChild(xml.toXml(doc.createElement("testxml")));
  qOut << "here is the document: \n" << doc;
  qsave_text(xml.toXmlDocument("TestXml", "testxml"), "/tmp/test.xml");
  return doc.toString();
}

void test_read(const QString& s)
{
  TestXml xml;
  xml.value = 51;
  QDomDocument doc; doc.setContent(s);
  if (!xml.fromXml(doc.firstChildElement("testxml")))
    qFatal("Could not read `testxml' tag.");
  qOut << "here is the document: \n" << doc;
  qOut << "here is the read value: " << xml.value << "\n";
  if (xml.value != 42)
    qFatal("Test failed.");
}

int main()
{
  test_read(test_write());
  TestXml xml; xml.value = 75;
  { QFile f("/tmp/test.xml"); xml.fromXmlDocument(f, "testxml"); }
  if (xml.value != 42)
    qFatal("Test failed.");
  qOut << "here is the read value: " << xml.value << "\n";
  return 0;
}
