//
// egbis.h
// 
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef ACSEGMENTOR_egbis_H
# define ACSEGMENTOR_egbis_H

# include <ntk/core.h>
# include <ntk/vxl/vil_utils.h>

# include "segmentor.h"

/**
 * Implementation of the segmentation algorithm described in:
 * 
 * Efficient Graph-Based Image Segmentation
 * Pedro F. Felzenszwalb and Daniel P. Huttenlocher
 * International Journal of Computer Vision, 59(2) September 2004.
 *
 * Copy/paste from the authors' code, available at: 
 * http://people.cs.uchicago.edu/~pff/segment/
 */
vil_image_view<int32> egbis_segment(const ntk::vil_byte_image& im, 
                                     double sigma = 0.5, 
                                     double k = 500, 
                                     int min_size = 20);

class EgbisFunctor : public SegmentationFunctor
{
  public:
    EgbisFunctor(double sigma, double k, int min_size)
      : m_sigma(sigma), m_k(k), m_min_size(min_size)
    {}
  
    virtual String name() const 
    { 
      String s = "Egbis";
      s << "S" << m_sigma << "K" << m_k << "MS" << m_min_size;
      return s;
    }
  
    virtual vil_image_view<int32> operator()(const ntk::vil_byte_image& im)
    { return egbis_segment(im, m_sigma, m_k, m_min_size); }
  
  private:
    double m_sigma;
    double m_k;
    int m_min_size;
};

#endif // ndef ACSEGMENTOR_egbis_H
