//
// ac_application.h
//
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef ACSEGMENTOR_gui_ac_application_H
# define ACSEGMENTOR_gui_ac_application_H

#include <ntk/core.h>
#include <ntk/gui/per_thread_logger.h>
#include "main_component.h"

class MainWindow : public DocumentWindow
{
public:
  MainWindow() : DocumentWindow("Acsegmentor v0.1", Colours::grey,
    DocumentWindow::allButtons, true)
  {
    // setUsingNativeTitleBar(true);
    setResizable(true, false);
    centreWithSize(500, 550);

    MainComponent* main_component = new MainComponent();
    main_component->setVisible(true);
    setContentComponent(main_component);
  }

protected:
  virtual void closeButtonPressed()
  {
    JUCEApplication::getInstance()->systemRequestedQuit();
  }
};

class AcApplication : public JUCEApplication
{
public:
  AcApplication () : m_main_window(0)
  {}

  ~AcApplication()
  {}

  void initialise (const String& commandLine)
  {
    Logger::setCurrentLogger(PerThreadLogger::getInstance());
    m_main_window = new MainWindow();
    m_main_window->setVisible(true);
  }

  void shutdown()
  {
    deleteAndZero(m_main_window);
  }

  const String getApplicationName()
  {
    return "Acsegmentor";
  }

  const String getApplicationVersion()
  {
    return "0.1";
  }

  bool moreThanOneInstanceAllowed()
  {
    return true;
  }

  void anotherInstanceStarted (const String& commandLine)
  {
  }

private:
  DocumentWindow* m_main_window;
};

#endif // !ACSEGMENTOR_gui_ac_application_H
