/*
  ==============================================================================

  This is an automatically generated file created by the Jucer!

  Creation date:  12 Sep 2007 11:12:22 am

  Be careful when adding custom code to these files, as only the code within
  the "//[xyz]" and "//[/xyz]" sections will be retained when the file is loaded
  and re-saved.

  Jucer version: 1.11

  ------------------------------------------------------------------------------

  The Jucer is part of the JUCE library - "Jules' Utility Class Extensions"
  Copyright 2004-6 by Raw Material Software ltd.

  ==============================================================================
*/

//[Headers] You can add your own extra header files here...
#include "../ac_learn_process.h"
#include "../segmentor_factory.h"
#include "../ac_segmentor.h"
#include "../ac_selector.h"
#include "../ac_threshold_database.h"
//[/Headers]

#include <ntk/ntk_with_macros.h>
#include "learning_main_component.h"


//[MiscUserDefs] You can add your own user definitions and misc code here...
using namespace ntk;

class LearningJob : public Thread, public AsyncUpdater, public ChangeListener
{
  public:
    LearningJob(const PropertySetSubject& properties,
                LearningJobListener& listener)
      : Thread("Learning"),
      m_properties(properties),
      m_listener(listener),
      m_finished(false)
    {
      SegmentorFactory factory;
      try {
        m_learn_process = factory.createLearnProcess(m_properties);
      }
      catch (const vcl_exception& e)
      {
        m_error_message = String(T("Could not create learn process `"))
            << e.what() << "'";
        return;
      }
      if (!m_learn_process)
      {
        m_error_message = T("Could not load create segmentor.");
        return;
      }
      m_learn_process->setChangeListener(this);
    }

    virtual ~LearningJob()
    {
    }

  public:
    const String& getErrorMessage() const { return m_error_message; }
    bool isFinished() const { return m_finished; }
    
    vil_byte_image getCurrentSourceImage() const
    { return m_learn_process->getCurrentSourceImage(); }
    
    vcl_vector<double> getCurrentThresholds() const
    { return m_learn_process->getCurrentThresholds(); }

  public:
    bool isValid() const { return m_learn_process; }

    virtual void changeListenerCallback(void*)
    {
      // Change from the learning process.
      if (threadShouldExit())
      {
        m_learn_process->setShouldStop();
      }
      else
      {
        triggerAsyncUpdate();
      }
    }

    virtual void run()
    {
      if (!isValid())
        return stopWithError("No segmentor or no source image");

      m_learn_process->learn();

      if (threadShouldExit()) return;

      if (m_properties.getValue("threshold_database").isNotEmpty())
      {
        ntk_log() << "Saving threshold database...";
        String f = m_properties.getValue("threshold_database");
        m_learn_process->getSegmentor()->selector()->thresholdDatabase()->saveToFile(f);
        ntk_log() << "[OK]";
      }
      m_finished = true;
      triggerAsyncUpdate();
    }

  protected:
    void stopWithError(const String& message)
    {
      m_error_message = message;
      m_finished = true;
      triggerAsyncUpdate();
    }

    void handleAsyncUpdate()
    {
      if (threadShouldExit()) return;
      if (m_finished)
        m_listener.handleLearningFinished(this);
      else
        m_listener.handleLearningNewIteration(this);
    }

  private:
    const PropertySetSubject& m_properties;
    LearningJobListener& m_listener;
    AcLearnProcessPtr m_learn_process;
    String m_error_message;
    bool m_finished;
};
//[/MiscUserDefs]

//==============================================================================
LearningMainComponent::LearningMainComponent (PropertySetSubject& properties)
    : m_properties(properties),
      m_source_image (0),
      m_logger (0),
      m_plot (0)
{
  //[Constructor_pre] You can add your own custom stuff here..
  m_learning_job = 0;
  //[/Constructor_pre]

  addAndMakeVisible (m_source_image = new ImageComponent());
  m_source_image->setName (T("source component"));

  addAndMakeVisible (m_logger = new LoggerComponent ("[Launching segmentation]"));
  m_logger->setName (T("logger"));

  addAndMakeVisible (m_plot = new PlotComponent());


  //[UserPreSize]
  //[/UserPreSize]

  setSize (600, 400);

  //[Constructor] You can add your own custom stuff here..
  m_plot->setTooltip("Maximal values of Fseg (Y axis) observed so far for each size category (X axis)");
  //[/Constructor]
}

LearningMainComponent::~LearningMainComponent()
{
  //[Destructor_pre]. You can add your own custom destruction code here..
  if (m_learning_job)
  {
    PerThreadLogger::getInstance()->removeLogger(m_learning_job);
    deleteAndZero(m_learning_job);
  }
  //[/Destructor_pre]

  deleteAndZero (m_source_image);
  deleteAndZero (m_logger);
  deleteAndZero (m_plot);

  //[Destructor]. You can add your own custom destruction code here..
  //[/Destructor]
}

//==============================================================================
void LearningMainComponent::paint (Graphics& g)
{
    //[UserPrePaint] Add your own custom painting code here..
    //[/UserPrePaint]

    g.fillAll (Colours::white);

    //[UserPaint] Add your own custom painting code here..
    //[/UserPaint]
}

void LearningMainComponent::resized()
{
  m_source_image->setBounds(0, 0, proportionOfWidth(0.5000f), getHeight() - 100);
  m_logger->setBounds(0, getHeight() - 100, proportionOfWidth(1.0000f), 100);
  m_plot->setBounds((getWidth() / 2), 0, proportionOfWidth(0.5000f), getHeight() - 100);
  //[UserResized] Add your own custom resize handling here..
  // FIXME: automatically add a border.
  const int padding = 2;
  const int half_padding = 1;
  m_source_image->setBounds(padding, padding,
                            proportionOfWidth(0.5000f) - half_padding - padding,
                            getHeight() - 100 - padding - half_padding);
  m_plot->setBounds((getWidth() / 2) + half_padding, padding,
                    proportionOfWidth(0.5000f) - half_padding - padding,
                    getHeight() - 100 - padding - half_padding);
  m_logger->setBounds(padding, getHeight() - 100 + half_padding,
                      proportionOfWidth(1.0000f) - 2*padding, 100 - padding - half_padding);
  //[/UserResized]
}

//[MiscUserCode] You can add your own definitions of your custom methods or any other code here...
bool LearningMainComponent :: startLearning()
{
  m_learning_job = new LearningJob(m_properties, *this);
  if (m_learning_job->isValid())
  {
    PerThreadLogger::getInstance()->registerLogger(m_learning_job, m_logger);
    m_learning_job->startThread();
    return true;
  }
  else
  {
    AlertWindow::showMessageBox(AlertWindow::WarningIcon,
                                "Error !",
                                m_learning_job->getErrorMessage(),
                                "Ok");
  }
  return false;
}

void LearningMainComponent :: handleLearningFinished(LearningJob* job)
{
  handleLearningNewIteration(job);
  if (m_learning_job->getErrorMessage().isNotEmpty())
  {
    AlertWindow::showMessageBox(AlertWindow::WarningIcon,
                                "Error !",
                                m_learning_job->getErrorMessage(),
                                "Ok");
  }
}

void LearningMainComponent :: handleLearningNewIteration(LearningJob*)
{
  vil_byte_image im = m_learning_job->getCurrentSourceImage();
  if (im) m_source_image->setImage(vil_to_juce(im));

  // Convert threshold to a plottable version.
  vcl_vector<double> thresholds = m_learning_job->getCurrentThresholds();
  vcl_map<double,double> tmp;
  double ymax = 0;
  foreach_idx(i, thresholds)
  {
    if (thresholds[i] < 0) continue;
    tmp[i] = thresholds[i];
    ymax = vcl_max(ymax, thresholds[i]);
  }
  m_plot->setPlot(tmp, 0, thresholds.size(), 0, ymax);
}

void LearningMainComponent :: askForQuit()
{
  if (!m_learning_job) return;
  m_learning_job->stopThread(-1);
}
//[/MiscUserCode]


//==============================================================================
#if 0
/*  -- Jucer information section --

  This is where the Jucer puts all of its metadata, so don't change anything in here!

BEGIN_JUCER_METADATA

<JUCER_COMPONENT documentType="Component" className="LearningMainComponent" componentName=""
                 parentClasses="public Component, public LearningJobListener"
                 constructorParams="PropertySetSubject&amp; properties" variableInitialisers="m_properties(properties)"
                 snapPixels="8" snapActive="1" snapShown="1" overlayOpacity="0.330000013"
                 fixedSize="0" initialWidth="600" initialHeight="400">
  <BACKGROUND backgroundColour="ffffffff"/>
  <GENERICCOMPONENT name="source component" id="aadf8a554cdc5c67" memberName="m_source_image"
                    virtualName="" explicitFocusOrder="0" pos="0 0 50% 100M" class="ImageComponent"
                    params=""/>
  <GENERICCOMPONENT name="logger" id="33000e36bea3fd06" memberName="m_logger" virtualName=""
                    explicitFocusOrder="0" pos="0 0Rr 100% 100" class="LoggerComponent"
                    params="&quot;[Launching segmentation]&quot;"/>
  <GENERICCOMPONENT name="" id="55e3cc15319094e5" memberName="m_plot" virtualName=""
                    explicitFocusOrder="0" pos="0C 0 50% 100M" class="PlotComponent"
                    params=""/>
</JUCER_COMPONENT>

END_JUCER_METADATA
*/
#endif
