//
// segmentation_window.h
//
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef ACSEGMENTOR_segmentation_window_H
# define ACSEGMENTOR_segmentation_window_H

# include <ntk/core.h>

# include "segmentation_main_component.h"

class SegmentationWindow : public DocumentWindow
{
  public:
    SegmentationWindow(const PropertySetSubject& properties)
      : DocumentWindow("Acsegmentor v0.1 - Segmentation results", Colours::grey,
                       DocumentWindow::allButtons, true),
        m_properties(properties)
    {
      // setUsingNativeTitleBar(true);
      setResizable(true, false);
      setSize(800, 500);
      setCentreRelative(0.6f, 0.6f);

      m_main_component = new SegmentationMainComponent(m_properties);
      m_main_component->setVisible(true);
      setContentComponent(m_main_component);
    }

    virtual ~SegmentationWindow()
    {}

  public:
    void startSegmentation()
    {
      if (!m_main_component->startSegmentation())
        delete this;
    }

  protected:
    virtual void closeButtonPressed()
    {
      delete this;
    }

  private:
    PropertySetSubject m_properties;
    SegmentationMainComponent* m_main_component;
};

#endif // ndef ACSEGMENTOR_segmentation_window_H
